@@ -10,7 +10,6 @@ has_names <- function(x) {
 `%||%` <- function(x, y) if (is.null(x)) y else x
 
 string_values <- function(x) {
-
   # TODO: Throw a condition object that can be caught for debugging purposes
   x <- tryCatch(x, error = function(x) "")
   unique(x[nzchar(x)])
@@ -48,7 +47,10 @@ parse_size <- function(x) {
     return(as.numeric(x))
   }
 
-  stop(sprintf("`%s` is not a valid size:\n  Must be a positive integer.", as.character(nme)), call. = FALSE)
+  stop(
+    sprintf("`%s` is not a valid size:\n  Must be a positive integer.", as.character(nme)),
+    call. = FALSE
+  )
 }
 
 id_field <- function(id, field, default = NULL) {
@@ -71,12 +73,42 @@ isPatternValue <- function(val) {
   grepl("[%|_]", val)
 }
 
-getSelector <- function(key, value) {
+# Per ODBC spec, "%" is synonymous with
+# zero or more of any characters. "_" is
+# any one character.  Here we convert
+# a string with pattern value style wildcards
+# to one that can be used in a regex search.
+convertWildCards <- function(val) {
+  val <- gsub("%", "(.*)", val)
+  gsub("_", "(.)", val)
+}
+
+getSelector <- function(key, value, exact) {
+  if (is.null(value)) {
+    return("")
+  }
   comp <- " = "
-  if (isPatternValue(value)) {
+  if ((value == "%" || !exact) &&
+    isPatternValue(value)) {
     comp <- " LIKE "
   }
+  if (exact && (value != "%")) {
+    value <- escapePattern(value)
+  }
   value <- paste0("'", value, "'")
 
   paste0(" AND ", key, comp, value)
 }
+
+# Will iterate over charsToEsc argument and for each:
+# will escape any un-escaped occurance in `x`.
+escapePattern <- function(x, charsToEsc = c("_"), escChar = "\\\\") {
+  if (is.null(x) || inherits(x, "AsIs")) {
+    return(x)
+  }
+  matchGroup <- paste(charsToEsc, collapse = "|")
+  pattern <- paste0("([^", escChar, "])(", matchGroup, ")")
+  replace <- paste0("\\1", escChar, "\\2")
+  x <- gsub(pattern, replace, x)
+  I(x)
+}
